FarmingEssentials = {}

FarmingEssentials.initialized = false
FarmingEssentials.superStrengthEnabled = false  -- Initialize super strength state
FarmingEssentials.lastToggleTime = 0  -- Debounce timing to prevent rapid toggling
FarmingEssentials.isFlightEnabled = false
FarmingEssentials.flightSpeed = 1000 -- Flight speed in m/s
FarmingEssentials.verticalSpeed = 500 -- Vertical speed in m/s
FarmingEssentials.lastToggleTime = 0 -- Last toggle timestamp (for debounce)
FarmingEssentials.toggleCooldown = 500 -- Cooldown in milliseconds
FarmingEssentials.lastSuperStrengthToggleTime = 0
FarmingEssentials.lastFlightToggleTime = 0


function FarmingEssentials:keyEvent(unicode, sym, modifier, isDown)
    -- HUD Toggle
    if Input.isKeyPressed(Input.KEY_equals) then
        g_currentMission.hud:consoleCommandToggleVisibility()
    end

    -- Money(LCTRL + LALT + 0)
    if isDown and bitAND(modifier, Input.MOD_LCTRL) > 0 and bitAND(modifier, Input.MOD_LALT) > 0 and Input.isKeyPressed(Input.KEY_0) then
        g_currentMission:consoleCommandCheatMoney(10000)
    end  

    --Remove Money (LCTRL + LALT + 4)
    if isDown and bitAND(modifier, Input.MOD_LCTRL) > 0 and bitAND(modifier, Input.MOD_LALT) > 0 and Input.isKeyPressed(Input.KEY_4) then
        g_currentMission:consoleCommandCheatMoney(-10000)
    end 
    
    -- Sleep Anywhere
    if bitAND(modifier, Input.MOD_LCTRL) > 0 and bitAND(modifier, Input.MOD_LALT) > 0 and Input.isKeyPressed(Input.KEY_z) then
        g_sleepManager:showDialog()
    end
end


-- Toggle Super Strength
function FarmingEssentials:toggleSuperStrength()
    if g_currentMission.playerSystem ~= nil then
        local player = g_currentMission.playerSystem.players[g_currentMission.playerUserId]
        if player ~= nil and player.hands ~= nil then
            local success, errorMessage = pcall(function()
                player.hands:consoleCommandToggleSuperStrength()
            end)

            if success then
                FarmingEssentials.superStrengthEnabled = not FarmingEssentials.superStrengthEnabled
                print("Super Strength toggled. State:", FarmingEssentials.superStrengthEnabled)
            else
                print("Error toggling Super Strength:", errorMessage)
            end

            -- Synchronize with server/clients
            if g_server ~= nil then
                g_server:broadcastEvent(FarmingEssentialsEvent.new(FarmingEssentials.superStrengthEnabled), nil, nil)
            elseif g_client ~= nil then
                g_client:getServerConnection():sendEvent(FarmingEssentialsEvent.new(FarmingEssentials.superStrengthEnabled))
            end
        end
    end
end

-- Toggle Flight Mode
function FarmingEssentials:toggleFlightMode()
    if g_currentMission.playerSystem ~= nil then
        local player = g_currentMission.playerSystem.players[g_currentMission.playerUserId]
        if player ~= nil then
            FarmingEssentials.isFlightEnabled = not FarmingEssentials.isFlightEnabled
            print("Flight mode toggled. State:", FarmingEssentials.isFlightEnabled)
        end
    end
end

-- Get Active Camera Node
function FarmingEssentials:getActiveCameraNode()
    local cameraNode = getCamera()
    if cameraNode == nil then
        print("Error: No active camera node found.")
    end
    return cameraNode
end

-- Update Flight Movement
function FarmingEssentials:updateFlightMovement(dt)
    if not FarmingEssentials.isFlightEnabled then return end

    local player = g_currentMission.playerSystem.players[g_currentMission.playerUserId]
    if player == nil then return end

    local cameraNode = FarmingEssentials:getActiveCameraNode()
    if cameraNode == nil then return end

    local forwardX, forwardY, forwardZ = localDirectionToWorld(cameraNode, 0, 0, 1)
    local sideX, sideY, sideZ = localDirectionToWorld(cameraNode, 1, 0, 0)

    local moveX, moveY, moveZ = 0, 0, 0

    -- Forward and Backward
    if Input.isKeyPressed(Input.KEY_w) then
        moveX, moveY, moveZ = moveX - forwardX, moveY - forwardY, moveZ - forwardZ
    elseif Input.isKeyPressed(Input.KEY_s) then
        moveX, moveY, moveZ = moveX + forwardX, moveY + forwardY, moveZ + forwardZ
    end

    -- Left and Right
    if Input.isKeyPressed(Input.KEY_a) then
        moveX, moveY, moveZ = moveX - sideX, moveY - sideY, moveZ - sideZ
    elseif Input.isKeyPressed(Input.KEY_d) then
        moveX, moveY, moveZ = moveX + sideX, moveY + sideY, moveZ + sideZ
    end

    -- Ascend and Descend
    if Input.isKeyPressed(Input.KEY_e) then
        moveY = moveY - FarmingEssentials.verticalSpeed * dt * 0.001
    elseif Input.isKeyPressed(Input.KEY_q) then
        moveY = moveY + FarmingEssentials.verticalSpeed * dt * 0.001
    end

    -- Normalize movement to prevent faster diagonal movement
    local length = math.sqrt(moveX^2 + moveY^2 + moveZ^2)
    if length > 0 then
        moveX, moveY, moveZ = moveX / length, moveY / length, moveZ / length
    end

    -- Apply movement
    local posX, posY, posZ = getTranslation(player.rootNode)
    setTranslation(player.rootNode, posX + moveX * dt * 0.015, posY + moveY * dt * 0.015, posZ + moveZ * dt * 0.015)
end

-- Update Function
function FarmingEssentials:update(dt)
    local currentTime = g_currentMission.time

    -- Toggle Super Strength (ALT + S)
    if Input.isKeyPressed(Input.KEY_lalt) and Input.isKeyPressed(Input.KEY_s) then
        if currentTime - FarmingEssentials.lastSuperStrengthToggleTime > FarmingEssentials.toggleCooldown then
            FarmingEssentials.lastSuperStrengthToggleTime = currentTime
            FarmingEssentials:toggleSuperStrength()
        end
    end

    -- Toggle Flight Mode (ALT + F)
    if Input.isKeyPressed(Input.KEY_lalt) and Input.isKeyPressed(Input.KEY_f) then
        if currentTime - FarmingEssentials.lastFlightToggleTime > FarmingEssentials.toggleCooldown then
            FarmingEssentials.lastFlightToggleTime = currentTime
            FarmingEssentials:toggleFlightMode()
        end
    end

    -- Update flight movement
    FarmingEssentials:updateFlightMovement(dt)
end

-- Multiplayer Event Logic (unchanged)
FarmingEssentialsEvent = {}
FarmingEssentialsEvent_mt = Class(FarmingEssentialsEvent, Event)

function FarmingEssentialsEvent:emptyNew()
    return Event.new(FarmingEssentialsEvent_mt)
end

function FarmingEssentialsEvent.new(state)
    local self = FarmingEssentialsEvent:emptyNew()
    self.state = state
    return self
end

function FarmingEssentialsEvent:readStream(streamId, connection)
    self.state = streamReadBool(streamId)
    self:run(connection)
end

function FarmingEssentialsEvent:writeStream(streamId, connection)
    streamWriteBool(streamId, self.state)
end

function FarmingEssentialsEvent:run(connection)
    if g_currentMission.playerSystem ~= nil then
        local player = g_currentMission.playerSystem.players[g_currentMission.playerUserId]
        if player ~= nil and player.hands ~= nil then
            local success, errorMessage = pcall(function()
                player.hands:consoleCommandToggleSuperStrength()
            end)

            if not success then
                print("Error applying SuperStrength state on client:", errorMessage)
            end
        end
    end

    FarmingEssentials.superStrengthEnabled = self.state

    if not connection:getIsServer() then
        g_server:broadcastEvent(FarmingEssentialsEvent.new(self.state), nil, connection)
    end
end

-- Register the Event
InitEventClass(FarmingEssentialsEvent, "FarmingEssentialsEvent")


addModEventListener(FarmingEssentials)